#include <uart_cfg.h>
#include "WString.h"
#ifndef DFL168A_Async_h
#define DFL168A_Async_h


#define byte uint8_t

#define AUTO_PROTOCOL 0
#define ISO15765_11_500_PROTOCOL 6
#define ISO15765_29_500_PROTOCOL 7
#define ISO15765_11_250_PROTOCOL 8
#define ISO15765_29_250_PROTOCOL 9
#define J1939_PROTOCOL 10
#define USER1_PROTOCOL 11
#define USER2_PROTOCOL 12
#define J1708_PROTOCOL 13

#define WAITING 1
#define SUCCESS 2
#define FAIL 0


class PGN65256Class
{
  public:
   class DFL168A * Pareant;
   PGN65256Class();
   PGN65256Class(class DFL168A * Pareant);
   unsigned char refresh();
   bool getAltitude(float &Altitude);
   bool getNavBasedSpeed(float &Speed);
  private:
   bool SuccessFresh;
   String FreshResultStr;
};

class PGN65269Class
{
  public:
   class DFL168A * Pareant;
   PGN65269Class();
   PGN65269Class(class DFL168A * Pareant);
   unsigned char refresh();
   bool getBarometricPressure(float &BaroPressure);
   bool getAmbientTemp(int &AmbientTemp);
   bool getInletTemp(int &InletTemp);
   bool getRoadTemp(int &RoadTemp);
   bool getCabInteriorTemp(int &CabInteriorTemp);
  private:
   bool SuccessFresh;
   String FreshResultStr;
};

class PGN65257Class
{
  public:
   class DFL168A * Pareant;
   PGN65257Class();
   PGN65257Class(class DFL168A * Pareant);
   unsigned char refresh();
   bool getEngineTripFuel(float &EngineTripFuel);
   bool getEngineTotalFuelUsed(float &EngineTotalFuelUsed);
  private:
   bool SuccessFresh;
   String FreshResultStr;
};

class PGN61444Class
{
  public:
   class DFL168A * Pareant;
   PGN61444Class();
   PGN61444Class(class DFL168A * Pareant);
   unsigned char refresh();
   bool getActualEngineTorque(int &ActualEngineTorque);  //-125 to +125 (%)
   bool getEngineSpeed(int &EngineSpeed);
  private:
   bool SuccessFresh;
   String FreshResultStr;
};

class PGN61443Class
{
  public:
   class DFL168A * Pareant;
   PGN61443Class();
   PGN61443Class(class DFL168A * Pareant);
   unsigned char refresh();
   bool getAccelPedalPosi1(float &AccelPedalPosi1);  //0 to 100 (%)
   bool getAccelPedalPosi2(float &AccelPedalPosi2);  //0 to 100 (%)
   bool getEnginePerLoadAtCurrSpeed(int &EnginePerLoadAtCurrSpeed);  //0 to 125 (%)
  private:
   bool SuccessFresh;
   String FreshResultStr;
};

class PGN65270Class
{
  public:
   class DFL168A * Pareant;
   PGN65270Class();
   PGN65270Class(class DFL168A * Pareant);
   unsigned char refresh();
   bool getIntakeManifoldPressure(int &IntakeManifoldPressure);   //0-500kpa 2kPa/Bit, BoostPressure=IntakeManifold1# pressure
   bool getIntakeManifoldTemp(int &IntakeManifoldTemp);  //-40 to 210, 1 degree c/bit
   bool getEngineAirInletPressure(int &EngineAirInletPressure);  //0-500kPa, 2kPa/bit
   bool getEngineExhaustGasTemp(int &EngineExhaustGasTemp);  //-273 to 1734.96875  0.03125C/bit
   bool getEngineAirFilterDiffPressure(float &EngineAirFilterDiffPressure);  //0 to 12.5kPa, 0.05kPa/bit (%)
  private:
   bool SuccessFresh;
   String FreshResultStr;
};
class PGN65271Class
{
  public:
   class DFL168A * Pareant;
   PGN65271Class();
   PGN65271Class(class DFL168A * Pareant);
   unsigned char refresh();
   bool getAlternatorVoltage(float &AlternatorVoltage);   //0.05V/Bit
   bool getElectricalVoltage(float &ElectricalVoltage);  
   bool getBatteryVoltage(float &BatteryVoltage); 
  private:
   bool SuccessFresh;
   String FreshResultStr;
};
class PGN65272Class
{
  public:
   class DFL168A * Pareant;
   PGN65272Class();
   PGN65272Class(class DFL168A * Pareant);
   unsigned char refresh();
   bool getTransmissionOilLevel(float &Percent);
   bool getTransmissionOilLevelHighLow(float &HighLow);
   bool getTransmissionOilPressure(float &Pressure); //0 to 4000 kPa, 16 kPa/bit, 4th posion
   bool getTransmissionOilTemp(float &Temperature);//-273 to 1734.96875 deg C, 0.03125 deg C/bit, 5th and 6th
  private:
   bool SuccessFresh;
   String FreshResultStr;
};
class PGN65266Class
{
  public:
   class DFL168A * Pareant;
   PGN65266Class();
   PGN65266Class(class DFL168A * Pareant);
   unsigned char refresh();
   bool getFuelRate(float &FuelRate);   
   bool getInstantFuelEconomy(float &InstantFuelEconomy);  
   bool getAvgFuelEconomy(float &AvgFuelEconomy); 
   bool getEngineThrottlePos(float &EngineThrottlePos); 
   
  private:
   bool SuccessFresh;
   String FreshResultStr;
};

class PGN65263Class
{
  public:
   class DFL168A * Pareant;
   PGN65263Class();
   PGN65263Class(class DFL168A * Pareant);
   unsigned char refresh();
   bool getFueDeliveryPressure(int &FueDeliveryPressure);   
   bool getEngineOilLevel(float &EngineOilLevel);  
   bool getEngineOilPressure(int &EngineOilPressure); 
   bool getEngineCoolantPressure(int &EngineCoolantPressure); 
   bool getEngineCoolantLevel(float &EngineCoolantLevel); 
  private:
   bool SuccessFresh;
   String FreshResultStr;
};

class PGN65253Class
{
  public:
   class DFL168A * Pareant;
   PGN65253Class();
   PGN65253Class(class DFL168A * Pareant);
   unsigned char refresh();
   bool getTotalEngineHours(float &TotalEngineHours);   
   bool getTotalEngineRevolutions(float &TotalEngineRevolutions);     
  private:
   bool SuccessFresh;
   String FreshResultStr;
};

class PGN65214Class
{
  public:
   class DFL168A * Pareant;
   PGN65214Class();
   PGN65214Class(class DFL168A * Pareant);
   unsigned char refresh();
   bool getRatedEngineSpeed(float &RatedEngineSpeed);      
  private:
   bool SuccessFresh;
   String FreshResultStr;
};

class PGN65248Class
{
  public:
   class DFL168A * Pareant;
   PGN65248Class();
   PGN65248Class(class DFL168A * Pareant);
   unsigned char refresh();
   bool getTripDistance(float &TripDistance);   
   bool getTotalDistance(float &TotalDistance);      
  private:
   bool SuccessFresh;
   String FreshResultStr;
};

class PGN65276Class
{
  public:
   class DFL168A * Pareant;
   PGN65276Class();
   PGN65276Class(class DFL168A * Pareant);
   unsigned char refresh();
   bool getWasherFluidLevel(float &WasherFluidLevel);   
   bool getFuelLevel1(float &FuelLevel1);  
   bool getFuelLevel2(float &FuelLevel2);      
  private:
   bool SuccessFresh;
   String FreshResultStr;
};

class PGN65265Class
{
  public:
   class DFL168A * Pareant;
   PGN65265Class();
   PGN65265Class(class DFL168A * Pareant);
   unsigned char refresh();
   bool getWheelBasedVehicleSpeed(float &WheelBasedVehicleSpeed);  
   //Mar07 2019 add SPN70 and SPN597 start (version 1.0.3)
   bool getParkingBrake(bool &ParkingBreakSet);
   bool getBrake(bool &BreakPedalDepressed);
   //Mar07 2019 add SPN70 and SPN597 end
  private:
   bool SuccessFresh;
   String FreshResultStr;
};
class PGN57344Class
{
 public:
   class DFL168A * Pareant;
   PGN57344Class();
   PGN57344Class(class DFL168A * Pareant);
   unsigned char refresh();
   bool getSeatBelt(bool buckled); //fix bug Mar 07 2019
   bool getSeatBelt(bool &buckled);
 private:
   bool SuccessFresh;
   String FreshResultStr;
};
//*********************************************

class PGN65267Class
{
  public:
   class DFL168A * Pareant;
   PGN65267Class();
   PGN65267Class(class DFL168A * Pareant);
   unsigned char refresh();
   bool getLatitude(float &Latitude);
   bool getLongitude(float &Longitude);
  private:
   bool SuccessFresh;
   String FreshResultStr;
};

class PGN65262Class
{
  public:  
   class DFL168A * Pareant;
   PGN65262Class();
   PGN65262Class(class DFL168A * Pareant);
   unsigned char refresh();
   bool getCoolantTemperature(int &temp);
   bool getFuelTemp(int &temp);
   bool getOilTemp(int &temp);
  private:
   bool SuccessFresh;
   String FreshResultStr;
};

//version 1.0.2 add ---start
class PGN64996Class
{
public:
	class DFL168A * Pareant;
	PGN64996Class();
	PGN64996Class(class DFL168A * Pareant);
	unsigned char refresh();
	bool getPayLoad(int &PayLoad);	  //0 to 250%, 1%/bit, 0 offset
private:
	bool SuccessFresh;
	String FreshResultStr;
};

class PGN61445Class
{
public:
	class DFL168A * Pareant;
	PGN61445Class();
	PGN61445Class(class DFL168A * Pareant);
	unsigned char refresh();
	bool getCurrentGear(int &CurrentGear);   //-125 to 125 and 251,  1 gear value/bit, -125 offset, negative values are reverse gears, positive values are forward gears, zero is neutral. 251(0xFB) is park. 251 actual= 126, but I still use "251" for matching spec
	bool getSelectedGear(int &SelectedGear);   //-125 to 125 and 251,  1 gear value/bit, -125 offset, negative values are reverse gears, positive values are forward gears, zero is neutral. 251(0xFB) is park. 251 actual= 126, but I still use "251" for matching spec
private:
	bool SuccessFresh;
	String FreshResultStr;
};
//version 1.0.2 add ----end

//version 1.1.0 add --------start
class PGN65268Class
{
public:
	class DFL168A * Pareant;
	PGN65268Class();
	PGN65268Class(class DFL168A * Pareant);
	unsigned char refresh();
	bool getTirePressure(int &TirePressure);   //0 to 1000kPa,  4kPa/bit, 0 offset
	bool getTireTemperature(float &Temperature);   //-273 to 1734.96875 deg C, 0.03125 deg C/ Bit,   -273 deg C offset
	bool getTireLocation(int &Front2RearNumber, int &Left2RighNumber);   //Front2RearNumber is high order 4 bits, Left2RighNumber is low order 4 bits
	bool getTireValvePressureMonitor(int &TireValvePressureMonitor);   //0b 000 to 0b111: 000 extreme high pressure,.....
private:
	bool SuccessFresh;
	String FreshResultStr;
};

//version 1.1.0 add --------end

class J1939Class
{
  friend class DFL168A;
  public:
   class DFL168A * Pareant;
   J1939Class();
   J1939Class(long Baudrate,class DFL168A * Pareant );
   PGN65267Class PGN65267;
   PGN65262Class PGN65262;
   PGN65256Class PGN65256;
   PGN65269Class PGN65269;
   PGN65257Class PGN65257;
   PGN61444Class PGN61444;
   PGN61443Class PGN61443;
   PGN65270Class PGN65270;
   PGN65271Class PGN65271;
   PGN65272Class PGN65272;
   PGN65266Class PGN65266;
   PGN65263Class PGN65263;
   PGN65253Class PGN65253;
   PGN65214Class PGN65214;
   PGN65248Class PGN65248;
   PGN65276Class PGN65276;
   PGN65265Class PGN65265;
   PGN57344Class PGN57344;
   PGN64996Class PGN64996;//version 1.0.2 add
   PGN61445Class PGN61445;//version 1.0.2 add
   PGN65268Class PGN65268;  //version 1.1.0 add
   unsigned char getVIN(String &VIN);
   //bool GetDTC(String &DTC);
   //unsigned char getDTC(unsigned char &DTC_Num, long  (&SPN)[5], unsigned char (&FMI)[5], unsigned char (&CM)[5],unsigned char (&OC)[5], unsigned char DTCFormat=1 );
   unsigned char getDTC(unsigned char &DTC_Num, long  (&SPN)[5], unsigned char (&FMI)[5], unsigned char (&CM)[5],unsigned char (&OC)[5], unsigned char DTCFormat );
   unsigned char clearDTC();
  private:
    long Baudrate;
};

class J1708Class
{
  public:
   class DFL168A * Pareant;
   J1708Class();
   J1708Class(class DFL168A * Pareant);
   unsigned char getAirPressure(int &AirPressure);   //0.0 to 1055 kPa,4.14 kPa/bit, PID=7
   unsigned char getEngineOilPressure(int &EngineOilPressure);  //0.0 to 1020 kPa, 4kpa/bit, PID=19
   unsigned char getEngineCoolantPressure(int &EngineCoolantPressure);  //0.0 to 510 kPa ,  2 kPa/bit,pid=20
   unsigned char getFuelLevel1(float &FuelLevel1);  //Primary fuel, 0.0 to 127.5% 0.5%/bit pid=96
   unsigned char getFuelLevel2(float &FuelLevel2);   //righ side fuel, 0.0 to 127.5% 0.5%/bit pid=38
   unsigned char getBarometricPressure(float &Pressure);  //0.0 to 153.0 kPa, 0.6 kPa/bit, PID=48
   unsigned char getEngineThrottlePos(float &EngineThrottlePos); //0.0 to 102.0%, 0.4%/bit, pid=51
   unsigned char getWasherFluidLevel(float &WasherFluidLevel);  //0.0 to 127.5%, 0.5%/bit, pid=80
   unsigned char getVehicleSpeed(float &VehicleSpeed);  //0.0 to 205.2 km/h , 0.805 km/h/bit, pid=84
   unsigned char getAccelPedalPosi1(float &AccelPedalPosi1);  //0.0 to 102.0%,0.4%/bit, pid=91  If only one is available, please use this one
   unsigned char getAccelPedalPosi2(float &AccelPedalPosi2);  //0.0 to 102.0%,0.4%/bit, pid=29
   unsigned char getAccelPedalPosi3(float &AccelPedalPosi3);  //0.0 to 102.0%,0.4%/bit, pid=28
   unsigned char getEngineLoad(float &Percent);//0.0 to 127.5%, 0.5%/bit, pid=92
   unsigned char getEngineOilLevel(float &EngineOilLevel);  //0.0 to 127.5%.0.5%/BIT PID=98
   unsigned char getCoolantTemperature(int &temp);  //We still use °C even though definition: 0.0 to 255.0 °F, 1.0 °F, PID=110
   unsigned char getEngineCoolantLevel(float &EngineCoolantLevel); //0.0 to 127.5%, 0.5%/bit, PID=111
   unsigned char getTransmissionOilLevel(float &Percent);//0.0 to 127.5%, 0.5%/bit, pid=124
   unsigned char getTransmissionOilLevelHighLow(float &HighLow);//–60.6 to 60.1 L,0.473L/bit,PID=125
   unsigned char getTransmissionOilPressure(float &Pressure);//0.0 to 3516 kPa, 13.8 kPa/bit, pid=127
   unsigned char getTransmissionOilTemp(float &Temperature); //We still use °C, even though definition: –8192.00 to +8191.75 °F, 0.25 °F, PID=177
   unsigned char getPowerSpecificInstantFuelEconomy(float &Rate); //0.0 to 129.1 kW·h/L, 1.97 x 10–3 kW·h/L /Bit, PID=130
   unsigned char getAvgFuelRate(float &FuelRate);    //0.0 to 1.07665 L/s , 16.428 x 10–6 L/s/Bit, PID=133
   unsigned char getInstantFuelEconomy(float &InstantFuelEconomy); //0.0 to 108.835 km/L, 1.66072 x 10–3 km/L/Bit, PID=184
   unsigned char getAvgFuelEconomy(float &AvgFuelEconomy); //0.0 to 108.835 km/L, 1.66072 x 10–3 km/L/Bit, PID=185
   unsigned char getElectricalVoltage(float &BatteryVoltage); //0.0 to 3276.75 V, 0.05 V/Bit, PID=158, after switch
   unsigned char getRatedEnginePower(float &Power); //0.0 to 48869.4 kW, 0.745 kW/bit, PID=166
   unsigned char getBatteryVoltage(float &BatteryVoltage);  //0.0 to 3276.75 V, 0.05V/bit  PID=168
   unsigned char getAlternatorVoltage(float &AlternatorVoltage);   //0.0 to 3276.75 V,0.05V/Bit, PID=167
   unsigned char getAmbientTemp(int &AmbientTemp);  //We still use °C, even though definition:–8192.00 to +8191.75 °F,0.25 °F/Bit, PID=171
   unsigned char getCargoAmbientTemp(int &CargoTemp);  //We still use °C, even though definition:–8192.00 to +8191.75 °F, 0.25 °F, PID=169
   unsigned char getRoadTemp(int &RoadTemp);//We still use °C, even though definition:–320.0 to +317.5 °F, 2.5 °F/Bit, PID=79
   unsigned char getCabInteriorTemp(int &CabInteriorTemp);//We still use °C, even though definition:–8192.00 to +8191.75 °F, 0.25 °F, PID=170
   unsigned char getInletTemp(int &InletTemp);//We still use °C, even though definition:–8192.00 to +8191.75 °F, 0.25 °F, PID=172
   unsigned char getFuelTemp(int &temp); //We still use °C, even though definition:–8192.00 to +8191.75 °F, 0.25 °F, PID=174
   unsigned char getOilTemp(int &temp); //We still use °C, even though definition:–8192.00 to +8191.75 °F, 0.25 °F, PID=175
   unsigned char getCargoWeight(float &CargoW); //0.0 to 1166056.9 N, 17.792 N/bit PID=181
   unsigned char getEngineTripFuel(float &EngineTripFuel);  //0.0 to 31009.6 L, 0.473 L/Bit, PID=182
   unsigned char getEngineTotalFuelUsed(float &EngineTotalFuelUsed);
   unsigned char getFuelRate(float &FuelRate);    //0.0 to 1.07665 L/s , 16.428 x 10–6 L/s/Bit, PID=183
   unsigned char getRatedEngineSpeed(float &RatedEngineSpeed);   //0.0 to 16383.75 rpm, 0.25 rpm/bit, PID=189
   unsigned char getEngineSpeed(int &EngineSpeed);   //0.0 to 16383.75 rpm, 0.25rpm/Bit, pid=190
   
   unsigned char getIntakeManifoldTemp(int &IntakeManifoldTemp);  //We still use °C even though definition: 0.0 to 255.0 °F, 1.0 °F/Bit  PID=105
   unsigned char getPowerTakeoffStatus(bool &PTOModeActive, bool &ClutchSwitchOn, bool &BrakeSwitchOn, bool &AccelSwitchOn, bool &ResumeSwitchOn, bool &CoastSwitchOn, bool &SetSwitchOn, bool &PTOControlSwitchOn); //Bit-Mapped,PID=89
   unsigned char getTripDistance(float &TripDistance);   //0.0 to 691207984.6 km, 0.160934 km/bit, PID=244
   unsigned char getTotalDistance(float &TotalDistance); //0.0 to 691207984.6 km, 0.160934 km/bit, PID=245
   unsigned char getTotalEngineHours(float &TotalEngineHours); //0.0 to 214748364.8 h, 0.05h/bit, PID=247
   unsigned char getTotalEngineRevolutions(float &TotalEngineRevolutions);   //0 to 4294967295000 r.  1000 r/BIT, PID=249
   
     
   
   unsigned char getVIN(String &VIN);
   unsigned char getDTC(unsigned char &DTC_Num,unsigned char &MID,int (&PID_SID)[8],bool (&IsPID)[8], unsigned char (&FMI)[8],bool (& IsActive)[8],bool (&OccurrenceExist)[8],unsigned char (&OccurrenceCount)[8]);  //PID=194
   unsigned char clearDTC(unsigned char MID,int PID_SID, bool IsPID);  //PID=195, and clear all fault related to MID,and PID or SID
   unsigned char getFaultDescription(unsigned char MID,int PID_SID, bool IsPID,unsigned char FMI, String & FaultDescription); //PID=195,
   unsigned char getPIDSIDDescription(unsigned char MID,int PID_SID, bool IsPID, String & PID_SID_Description); //PID=195,
 private:
   unsigned char MID4Vehicle;
   unsigned char getbyte(char cmd[],long &Out1byte, bool Signed);
   unsigned char getPhysical_float(char cmd[],float &OutFloat, float scale,bool Signed);

};

class ISO15765Class
{
  friend class DFL168A;
  public:
   class  DFL168A * Pareant;
   ISO15765Class();
   ISO15765Class(class  DFL168A * Pareant);  
   unsigned char getCoolantTemperature(int &temp);
   unsigned char getEngineSpeed(int &EngineSpeed);
   unsigned char getVehicleSpeed(float &VehicleSpeed);
   unsigned char getIntakeManifoldPressure(int &IntakeManifoldPressure);   //0-255kpa 1kPa/Bit,
   unsigned char getFuelSystemStatus(bool &A_Openloop, bool &A_Closedloop,bool &A_OpenloopByDriving_Con,bool &A_OpenloopByFault,bool &A_ClosedloopButFault,
                            bool &B_Openloop, bool &B_Closedloop,bool &B_OpenloopByDriving_Con,bool &B_OpenloopByFault,bool &B_ClosedloopButFault);
   unsigned char getCalculatedLoadValue(int &CalculatedLoad);  //0 to 100%
   unsigned char getShortTermFuelTrimBank13(float &Bank1,float &Bank3); //-100 to 99.22%
   unsigned char getLongTermFuelTrimBank13(float &Bank1,float &Bank3); //-100 to 99.22%
   unsigned char getShortTermFuelTrimBank24(float &Bank2,float &Bank4); //-100 to 99.22%
   unsigned char getLongTermFuelTrimBank24(float &Bank2,float &Bank4); //-100 to 99.22%
   unsigned char getIgnitionTimingAdvance(float &Angle);  //-64 63.5 degree
   unsigned char getIntakeAirTemperature(int &temp);
   unsigned char getAirFlowRateFrmMAF(float &FlowRate);   //unit: g/s
   unsigned char getAbsThrottlePosition(float &Percent); //0 to 100%
   unsigned char getOxygenSensorLocation(bool &Bank1_Sensor1Present, bool &Bank1_Sensor2Present, bool &Bank1_Sensor3Present,bool &Bank1_Sensor4Present, bool &Bank3_Sensor1Present,bool &Bank3_Sensor2Present,
                                bool &Bank2_Sensor1Present, bool &Bank2_Sensor2Present, bool &Bank2_Sensor3Present,bool &Bank2_Sensor4Present, bool &Bank4_Sensor1Present,bool &Bank4_Sensor2Present);
   unsigned char getBank1OSensor1Voltage(float &OutVoltage);  // 0 to 1.275V
   unsigned char getBank1OSensor2Voltage(float &OutVoltage);  // 0 to 1.275V
   unsigned char getBank1OSensor3Voltage(float &OutVoltage);  // 0 to 1.275V
   unsigned char getBank1OSensor4Voltage(float &OutVoltage);  // 0 to 1.275V
   unsigned char getBank2OSensor1Voltage(float &OutVoltage);  // 0 to 1.275V
   unsigned char getBank2OSensor2Voltage(float &OutVoltage);  // 0 to 1.275V
   unsigned char getBank2OSensor3Voltage(float &OutVoltage);  // 0 to 1.275V
   unsigned char getBank2OSensor4Voltage(float &OutVoltage);  // 0 to 1.275V
   unsigned char getBank3OSensor1Voltage(float &OutVoltage);  // 0 to 1.275V
   unsigned char getBank3OSensor2Voltage(float &OutVoltage);  // 0 to 1.275V
   unsigned char getBank4OSensor1Voltage(float &OutVoltage);  // 0 to 1.275V
   unsigned char getBank4OSensor2Voltage(float &OutVoltage);  // 0 to 1.275V
   unsigned char getOBDCertified(String &OBD);
   unsigned char getTimeSinceEngineStart(unsigned int &TotalTime);
   unsigned char getDistanceTraveledMIL(unsigned int &Distance);    //Distance Traveled While MIL is  Activated
   unsigned char getFuelRailPressure(float &Pressure);    //kPa
   unsigned char getFuelLevelInput(float &Percent);  //%
   unsigned char getDistanceTraveledSinceDTC_Clear(unsigned int &Distance);  //Distance traveled since DTCs cleared
   unsigned char getBarometricPressure(int &Pressure);  //0 to 255 kPa
   unsigned char getControlModuleVoltage(float &Voltage);
   unsigned char getRelativeThrottlePosition(float &Percent);  //%
   unsigned char getAmbientTemp(int &AmbientTemp); //C
   unsigned char getCommandedThrottleActuatorControl(float &Percent);  //%
   unsigned char getEngineRunTimeMIL(unsigned int &TotalTime); //Minutes
   unsigned char getEngineRunTimeSinceDTC_Clear(unsigned int &TotalTime); //Minutes
   unsigned char getTypeOfFuelUsedCurrently(String & FuelType);  //Type of fuel currently being utilized by the vehicle
   unsigned char getRelativeAcceleratorPedalPosition(float &Percent);  //%
   unsigned char getHybridBatteryPackRemainingLife(float &Percent);  //%
   unsigned char getEngineOilTemperature(int &Tem); //C
   unsigned char getFuelRate(float &FuelRate);    //Engine Fuel Rate: L/h
   unsigned char getActualEngineTorque(int &ActualEngineTorque);  //-125 to +130 (%)
   unsigned char getMILStatus(bool &MIL_IS_ON);
   unsigned char getEngineRunTime(unsigned long &TotalEngineRunTime, unsigned long &TotalIdleRunTime,unsigned long &TotalRunTimeWithPTOActive);  //Time unit: Seconds,  PID 7F, their value must not zero, it means it does not support if zero
    
   unsigned char getVIN(String &VIN);
   unsigned char getDTC(unsigned char &DTC_Num,String ( &DTC)[8]);
   unsigned char clearDTC();
  private:

    bool PID_13support;
    bool PID_1DSupport;
    unsigned char get1byte(char cmd[],long &Out1byte);
    unsigned char get2byte(char cmd[],long &Out2byte);
    unsigned char getPhysical2byte_float(char cmd[],float &OutFloat, float offset, float scale);
    unsigned char getPhysicalbyte_float(char cmd[],float &OutFloat, float offset, float scale);
};

class GPSClass
{
  friend class DFL168A;
  public:
   class DFL168A * Pareant;
   GPSClass();
   GPSClass(long Baudrate, int Timeout,class DFL168A * Pareant );    
   unsigned char getGPSinfo(float &Latitude /*degree*/,float &Longitude, int &Speed/*Km/h*/, String &Time /*hh:mm::ss*/, String &Date /*dd/mm/yyyy*/);
   unsigned char getAltitude(float &Altitude /*Meter*/);
  
  private:
    unsigned char SendGPSCMD(const unsigned char * CMD);
    long Baudrate;
    int Timeout;   
      
};

class DFL168A
{
   friend class GPSClass;
   friend class ISO15765Class;
   friend class J1708Class;
   friend class J1939Class;
   friend class PGN65267Class;
   friend class PGN65262Class;
   friend class PGN65256Class;
   friend class PGN65269Class;
   friend class PGN65257Class;  
   friend class PGN61444Class; 
   friend class PGN61443Class; 
   friend class PGN65270Class;
   friend class PGN65271Class;
   friend class PGN65272Class;
   friend class PGN65266Class;
   friend class PGN65263Class;
   friend class PGN65253Class;
   friend class PGN65214Class;
   friend class PGN65248Class;
   friend class PGN65276Class;
   friend class PGN65265Class;
   friend class PGN57344Class;
   friend class PGN64996Class;   //version 1.0.2 add
   friend class PGN61445Class;    //version 1.0.2 add
   friend class PGN65268Class;    //version 1.1.0 add
   public:
    DFL168A();
    DFL168A(unsigned char currProtocol,int Timeout, long J1939Baudrate, long DEV_Baudrate, int DEV_timeout);
    bool begin(bool intrude,bool Fast);
    void end();
    void SerialPoll();
    J1939Class J1939;
    J1708Class J1708;
    ISO15765Class ISO15765;
    GPSClass GPS; 
   
    unsigned char getOnewireID(unsigned char (&ID)[7]);
    byte getDIN(int portNo, bool & Value);   //Jan 15 2021 STM32 for Async version
    byte setDOUT(int portNo, bool Value);  //Jan 15 2021 STM32 for Async version
    byte getAnalog(float & ADValue);  //Jan 15 2021 STM32 for Async version
    void beginTransparentSerial();
    void endTransparentSerial();
    bool setExitTransparentKey(unsigned char EndTransparentChar);  //must after begin()
    bool serialPortAvailable();  //if TransparentSerialAvailable is true, it will return true

	//version1.0.1 add--start
	bool setSleepDelay(unsigned int SleepDelayms);
	//version1.0.1 add--End


   private:
	HardwareSerial * mySerial;
    unsigned char ATCommand(const unsigned char * CMD);
    void HandleResponse(String &Resp);
    unsigned char DigitalCommand(const unsigned char * CMD);
    void clearRXDBuffer();

    unsigned char currentProtocol;
    bool TransparentSerialAvailable;  
    unsigned char EndTransparentChar;
    uint32_t Timeout;
    bool SuccessCurrentProtocol;  // it is true When  begin() return true,Or  setCuurentProtocol(unsigned char cuurProtocol)return true
    String ReturnStr;
};
#endif


