/*
 * myPriv.h
 *
 *  Created on: Dec. 22, 2020
 *      Author: Jack Xia Laptop
 */
extern "C"
{
   #include "stm32f4xx_hal.h"
}

#include "WString.h"

#define UART_Num 2   //Uart number used for DFL168A


#ifndef INC_MYPRIV_H_
#define INC_MYPRIV_H_

#if UART_Num==1
  extern UART_HandleTypeDef huart1;
#elif  UART_Num==2
  extern UART_HandleTypeDef huart2;
#elif  UART_Num==3
  extern UART_HandleTypeDef huart3;
#elif  UART_Num==4
  extern UART_HandleTypeDef huart4;
#endif


class HardwareSerial
{
   friend void HAL_UART_RxCpltCallback(UART_HandleTypeDef *huart);
   friend void HAL_UART_TxCpltCallback(UART_HandleTypeDef *huart);
   friend class DFL168A;
   friend class GPSClass;
public:
	HardwareSerial(UART_HandleTypeDef * huart);
	uint8_t available();
	String readStringUntil(unsigned char ch);
	bool peek(unsigned char ch);
	void print(const unsigned char * str);
	void println(const unsigned char * str);
	int16_t read();  //-1 denote no data
	//void begin(int baudrate);
	void begin();   //because we have baudrate in HUART
	void setTimeout(uint32_t to);
	void write(unsigned char ch);
	String readString();
	void end();
	void poll();


private:
	//Dec 30 2020 Modify,  we used UART_HandleTypeDef not pointer, at begining, we didn't call huart initilization, so we copy zero value,,------start
	//UART_HandleTypeDef huart;
	UART_HandleTypeDef * phuart;
	//Dec 30 2020 Modify,  we used UART_HandleTypeDef not pointer, at begining, we didn't call huart initilization, so we copy zero value,,------end
	unsigned char TxRingBuffer[256];
	unsigned char Tx_Head,Tx_Tail;

	unsigned char RxRingBuffer[256];
	unsigned char Rx_Head,Rx_Tail;

	unsigned char RxCplt;
    unsigned char TxCplt;
	uint8_t Index4Rx_insdeHAL;  //because of 256, maximun 255 due to ring buffer

	unsigned char TxBuffer4HAL[255];
	unsigned char RxBuffer4HAL[255];

	uint32_t TimeOut;

	bool RxRingBufferFull();
	bool RxRingBufferEmpty();


	bool TxRingBufferFull();
	bool TxRingBufferEmpty();



};

#ifdef IMPLEMENT4UARTCFG
   #if UART_Num==1
     class HardwareSerial  Serial1(&huart1);
   #elif  UART_Num==2
     class HardwareSerial  Serial2(&huart2);
   #elif  UART_Num==3
     class HardwareSerial  Serial3(&huart3);
   #elif  UART_Num==4
     class HardwareSerial  Serial4(&huart4);
   #endif

#else
   #if UART_Num==1
     extern class HardwareSerial  Serial1;
   #elif  UART_Num==2
     extern class HardwareSerial  Serial2;
   #elif  UART_Num==3
     extern class HardwareSerial  Serial3;
   #elif  UART_Num==4
     extern class HardwareSerial  Serial4;
   #endif
#endif

uint32_t millis();
void HAL_UART_RxCpltCallback(UART_HandleTypeDef *huart);
void HAL_UART_TxCpltCallback(UART_HandleTypeDef *huart);
bool isHexadecimalDigit(const unsigned char ch);
bool isSpace(const unsigned char ch);
void delay(uint32_t d);
#endif /* INC_MYPRIV_H_ */
