
#include "DFL168A_Async.h"
DFL168A::DFL168A() : J1939(250000,this), J1708(this),ISO15765(this),GPS(9600,500,this) {

  #if UART_Num==1
	mySerial=&Serial1;
   #elif  UART_Num==2
	mySerial=&Serial2;
   #elif  UART_Num==3
	mySerial=&Serial3;
   #elif  UART_Num==4
	mySerial=&Serial4;
   #endif

  currentProtocol=AUTO_PROTOCOL;
  TransparentSerialAvailable=false;
  EndTransparentChar=0x1b;
  Timeout=500;  //time out is 500ms;    
  SuccessCurrentProtocol=false;
} 


//DFL168A::DFL168A(UART_HandleTypeDef * XX) : J1939(), J1708(),ISO15765(),GPS() {
//  mySerial=XX;
//}

//DFL168A::DFL168A(byte SerialNo, byte currProtocol, int Timeout,long J1939Baudrate, long DEV_Baudrate): J1939(J1939Baudrate),J1708(),ISO15765(),GPS(DEV_Baudrate) {
DFL168A::DFL168A(uint8_t currProtocol, int Timeout,long J1939Baudrate, long DEV_Baudrate, int DEV_timeout): J1939(J1939Baudrate,this),J1708(this),ISO15765(this),GPS(DEV_Baudrate,DEV_timeout,this) {
 

  this->currentProtocol=currProtocol;
  TransparentSerialAvailable=false;
  EndTransparentChar=0x1b;
  this->Timeout=Timeout;  //time out is 500ms;
  SuccessCurrentProtocol=false;
  //mySerial=huart;
  #if UART_Num==1
	mySerial=&Serial1;
   #elif  UART_Num==2
	mySerial=&Serial2;
   #elif  UART_Num==3
	mySerial=&Serial3;
   #elif  UART_Num==4
	mySerial=&Serial4;
   #endif
  
}
void DFL168A::SerialPoll()
{
	mySerial->poll();
}
 uint8_t DFL168A::ATCommand(const unsigned char * CMD)
 {
    byte temp; 
    static byte ReturnValue=SUCCESS;
    static uint32_t tick4Read;   //Jan 10 2021 for peek '>'
    if (WAITING!=ReturnValue) {tick4Read=millis();mySerial->println(CMD); }
    if (0==mySerial->available()) {ReturnValue=WAITING; return WAITING;}
    //Jan 10 2021 for peek '>'-------start
    if (!mySerial->peek('>'))
    {
    	if (millis()-tick4Read<mySerial->TimeOut)
    	{
    		ReturnValue=WAITING; return WAITING;
    	}
    	else
    	{
    		mySerial->Rx_Head=mySerial->Rx_Tail;   //clear Receiving buffer
    		ReturnValue=FAIL;
    		return ReturnValue;
    	}
    }
    //Jan 10 2021 for peek '>'-------end
    ReturnStr=mySerial->readStringUntil('>');   
    
    //if  (ReturnStr=="") {ReturnValue=WAITING; return WAITING;}         
    //Serial.print(ReturnStr);     //debug     
    temp= (ReturnStr.length());              
    if (temp<6) 
    {
       //Serial.println("WRONG:0");
       //SuccessCurrentProtocol=false;         because may return is not OK, such as "ATI"
       ReturnValue=FAIL;
       return ReturnValue;    
    }
    if (ReturnStr.substring(temp-6,temp-4)!="OK")
    {
       //Serial.println(ReturnStr.substring(temp-6,temp-4));
      // SuccessCurrentProtocol=false;        
       ReturnValue=FAIL;
       return ReturnValue; 
    }
    else
    {
      //Serial.println("correct");  
      ReturnValue=SUCCESS;
      return ReturnValue; 
    }                      
              
    
 }
uint8_t DFL168A::DigitalCommand(const unsigned char * CMD)
{
   int i, len; 
   char tempChar;  
   static byte ReturnValue=SUCCESS;
   static uint32_t tick; //only ISO 15765 Needed, it's for minimum time interval of request
   static bool Sent=false; //only ISO 15765 Needed

   static uint32_t tick4timeout;
   
   if (!SuccessCurrentProtocol) {
	   //Serial.println("FAIL PROTOCOL"); 
	   ReturnStr=""; ReturnValue=FAIL; return ReturnValue;}
   i=0;
   while (CMD[i])
   {
     if (!isHexadecimalDigit(CMD[i])) {
       if (!isSpace(CMD[i])) {
		   //Serial.println("WRONG CMD");
		   ReturnStr=""; ReturnValue=FAIL; return ReturnValue;}
     }     
     i++;
   }
   if ((J1939_PROTOCOL==currentProtocol)||(J1708_PROTOCOL==currentProtocol))  
   {
      if (WAITING!=ReturnValue) { 
		  //Serial.print("Digital Cmd ");  Serial.print(CMD); Serial.println( " start, response is:"); 
		  tick4timeout=millis(); mySerial->println(CMD); }
   }
   else
   {
      //only ISO15765, Keep request minimum time interval is 150ms.
      if (ReturnValue!=WAITING) {tick=millis(); ReturnValue=WAITING; return ReturnValue;}
      //if ((millis()-tick)>149) //142,bad, 146ok
      if ((millis()-tick)>259u) //142,bad, 146ok, stm32 : 150 not work,159 ok
      {
         if (!Sent) { 
			 Sent=true; 
			 //Serial.print("Digital Cmd ");  Serial.print(CMD); Serial.println( " start, response is:"); 
			 tick4timeout=millis();
			 clearRXDBuffer();
			 mySerial->println(CMD);  //write
			 mySerial->poll();  //write immediatly
         }
      }
	  else
	  {
		  return WAITING;
	  }
   }  
   
   //if (0==mySerial->available()) {ReturnValue=WAITING; return WAITING;}       
   //if (mySerial->available()<=3) {ReturnValue=WAITING; return WAITING;} 
   if (mySerial->available()<=3)
   {
     if (millis()-tick4timeout<=(Timeout+100u))
     {
    	mySerial->poll();
        ReturnValue=WAITING;
        return WAITING;
     }
     else
     {
        //time out
        while (mySerial->available()) mySerial->read();  //clear buffer
        ReturnValue=FAIL;
        Sent=false;  //STM32 : Jan 08/2021 add for initial condition
        return FAIL;
     }
     
   }

   //Jan 10 2021 for peek '>'-------start
   if (!mySerial->peek('>'))
   {
     if (millis()-tick4timeout<Timeout+100u)
     {
       	ReturnValue=WAITING; return WAITING;
     }
     else
     {
    	ReturnStr="";
    	mySerial->Rx_Head=mySerial->Rx_Tail;   //clear Receiving buffer
    	Sent=false;    //STM32 : Jan 15/2021 add for initial condition
       	ReturnValue=FAIL;
       	return ReturnValue;
     }
   }
   //Jan 10 2021 for peek '>'-------end

   ReturnStr=mySerial->readStringUntil('>');       
   //if  (ReturnStr=="") {ReturnValue=WAITING; return ReturnValue; }
   Sent=false;
   //Serial.print(ReturnStr);     //debug  
   len=ReturnStr.length();
   for (i=0;i<len;i++)
   {
      tempChar=ReturnStr.charAt(i);
      if (!isHexadecimalDigit(tempChar))
      {
        if ((tempChar!=':') && (!isSpace(tempChar)) && (tempChar!='\r') &&(tempChar!='\n') )
        {
          //Serial.println("Digital Cmd fail");
          ReturnStr="";           
          ReturnValue=FAIL; return ReturnValue;
        }
      }
     
   }

   //remove /r/n at begining       
   if (('\n'==ReturnStr.charAt(0))||('\r'==ReturnStr.charAt(0))) ReturnStr.remove(0,1);
   if (('\n'==ReturnStr.charAt(0))||('\r'==ReturnStr.charAt(0))) ReturnStr.remove(0,1);  //second character become first
   //Serial.println("Digital Cmd success");
   ReturnValue=SUCCESS; return ReturnValue;
}
/*******************************************************************************
 *  It will process Digital command response
 *  Arguments:
 *  Input/output: Resp is response String, no \r\n for begining, but it has \r\n at end
 *                Resp will be output of handle, It will be pure resonse String no matter long frame or short frame,
 *                And it won't contains any \r\n
 *  Author: Jack Xia, Dafulai Electronics Inc, Dec 28 2017              
 ******************************************************************************/
void DFL168A::HandleResponse(String &Resp)
{
  char tempChar[64];
  String TempStr="";
  char tempChar9[17];
  int i,j,k;
  byte DataLen;
  char HexValue[]="0x000";
  bool longFrame=false;
  Resp.toCharArray(tempChar, 64);

  for (i=0;i<((strlen(tempChar)<23)?strlen(tempChar):23);i++)  //8 hex=16 bytes+2 (\r\n)+3(frame no)+1 (:)=22
  {
    if (':'==tempChar[i]) { longFrame=true; break; }
  }

  if (longFrame=false) 
  {
    for (i=0;i<strlen(tempChar);i++)
    {
      if ('\r'==tempChar[i]||'\n'==tempChar[i])
      {
        tempChar[i]='\0';
        TempStr+=tempChar;
        Resp=TempStr;
        return;
      }
    }
  }
  // so long frame
 HexValue[2]=tempChar[0];
 HexValue[3]=tempChar[1];
 HexValue[4]=tempChar[2];  //HexValue[0]='0' HexValue[1]='x'  HexValue[5]='\0'
 DataLen=strtol(HexValue,NULL,16);

  k=0; //bytes QTY
  j=0;
  for (i=0;i<strlen(tempChar);i++)
  {
    
    if (':'==tempChar[i])
    {
      //find ":"
      while (' '==tempChar[i+1]) 
      {
        i++;
      }  //remove space
      for (j=i+1;j<16+i+1;j++)
      {
        //one row handle
        if (('\r'!=tempChar[j])&&('\n'!=tempChar[j])){
          // NOt the \r\n
          tempChar9[j-(i+1)]=tempChar[j];
          k++;
          if (DataLen*2==k) 
          {
            tempChar9[j-(i+1)+1]='\0';
            TempStr+=tempChar9;
            Resp=TempStr;
            return;
          }
        }
        else
        {
          // \r\n
          tempChar9[j-(i+1)]='\0';
          TempStr+=tempChar9;
          if (DataLen*2==k) 
          {
            Resp=TempStr;
            return;
          } else   break;
        }
        
      }  // end ""one row"           
    }  //end of "find ":""
    
  }
  
}

bool DFL168A::begin(bool intrude=true,bool Fast=false) {
    char bufferx[50];
    byte temp;
    char i;
    String TempStr;
    String ResponseStr;
    long MyLong;
    //Serial.begin(57600);  //debug
    //delay(75);  // wait for "searing and version pass"   //version1.0.1

    mySerial->begin();
    mySerial->setTimeout(Timeout+50);
    #if 0   //Jan 08 2021:   Too small for STM32, //For ISO15765 STM32, Change to 80
    mySerial->setTimeout(50);
    #else
    mySerial->setTimeout(200);  //80,195 fail, 200ok
    #endif
    //Dec 30 2020 because our begin method only is called once,so no chance to call poll  --start
	//while (WAITING == (ATCommand((const unsigned char *)"AT Z")));   //repower for sync  //version1.0.1
	while (WAITING == (ATCommand((const unsigned char *)"AT Z"))) mySerial->poll();
	//Dec 30 2020 because our begin method only is called once,so no chance to call poll  --end
	delay(75);  // wait for "searching and version pass"   //version1.0.1

    while (mySerial->available()) mySerial->read();
   
    while (WAITING==ATCommand((const unsigned char *)"ATi")) mySerial->poll();
    temp=ReturnStr.length();
    if ((temp<17)||(ReturnStr.substring(temp-17,temp-10)!="DFL168A"))
    {    
        //Only DFL168A working   
        SuccessCurrentProtocol=false;
        return false;     
    }
    if (Fast)
    {
       //Serial.println("Fast begin!");
       SuccessCurrentProtocol=true;
       if (!intrude)
       {
           //while(ATCommand((const unsigned char *)"AT INTRUDE 0")!=SUCCESS);
    	   while(ATCommand((const unsigned char *)"AT INTRUDE 0")!=SUCCESS) mySerial->poll();
       }
       if ((AUTO_PROTOCOL==currentProtocol)||
        (ISO15765_11_500_PROTOCOL==currentProtocol)||
        (ISO15765_29_500_PROTOCOL==currentProtocol)||
        (ISO15765_11_250_PROTOCOL==currentProtocol)||
        (ISO15765_29_250_PROTOCOL==currentProtocol)) {
        bufferx[0]='0';
        bufferx[1]='1'; 
        bufferx[2]='0';
        bufferx[3]='0'; 
        bufferx[4]='\0';
        if (ISO15765.get2byte(/*char cmd[]*/bufferx,/*long &Out2byte*/MyLong)) //"0100" Command
        {
          if (MyLong&0x00002000) ISO15765.PID_13support=true; else ISO15765.PID_13support=false;  //PID 13 (Hex19) support, Bit31 is PID1, Bit 30 is PID2,..., Bit 0 is PID 32
          if (MyLong&0x00000008) ISO15765.PID_1DSupport=true; else ISO15765.PID_1DSupport=false;  //PID 1D (Hex29) support, Bit31 is PID1, Bit 30 is PID2,..., Bit 0 is PID 32
        }
       }
       //mySerial->setTimeout(1);
       return true;
    }
    TempStr=String("AT SP");
    if ((currentProtocol!=J1939_PROTOCOL)||((J1939_PROTOCOL==currentProtocol)&&(250000==J1939.Baudrate))) {

        TempStr+=String(currentProtocol,16);
        TempStr.toCharArray(bufferx, 50);
        while (WAITING==(temp=ATCommand((const unsigned char *)bufferx))) mySerial->poll();
        if (FAIL==temp)            
        {          
           SuccessCurrentProtocol=false;
           //Serial.println("Fail  to AT SP");
           return false;  
        }
    }
    else {
        //J1939 but baudrate is not 250000
        TempStr+="B";
        TempStr.toCharArray(bufferx, 50);
        while (WAITING==(temp=ATCommand((const unsigned char *)bufferx))) mySerial->poll();
        if (FAIL==temp)            
        {
           SuccessCurrentProtocol=false;
           return false;  
        }
        //set baud rate
        TempStr=String("AT PP 34 SV");
        //set baud rate
        //Baud rate=500k/setting value, so setting value=500000/baud rate
        temp=(byte)(500000/J1939.Baudrate);
        if (temp<=15) 
        {
          TempStr+="0";
        }

        TempStr+=String(temp,16);
        TempStr.toCharArray(bufferx, 50);
        while (WAITING==(temp=ATCommand((const unsigned char *)bufferx))) mySerial->poll();
        if (FAIL==temp)            
        {
           SuccessCurrentProtocol=false;
           return false;  
        }
        while (WAITING==(temp=ATCommand((const unsigned char *)"AT PP 34 ON"))) mySerial->poll();
        if (FAIL==temp)             
        {
           SuccessCurrentProtocol=false;
           return false;  
        }   
        while (WAITING==(temp=ATCommand((const unsigned char *)"AT PP 33 SV 02"))) mySerial->poll();  //j1939 format
        if (FAIL==temp)
        {
           SuccessCurrentProtocol=false;
           return false;  
        }   
        while (WAITING==(temp=ATCommand((const unsigned char *)"AT PP 33 ON"))) mySerial->poll();
        if (FAIL==temp)
        {
           SuccessCurrentProtocol=false;
           return false;  
        } 
        while (WAITING==(temp=ATCommand((const unsigned char *)"AT PPP"))) mySerial->poll();
        if (FAIL==temp)  
        {
           SuccessCurrentProtocol=false;
           return false;  
        }              
       
    }
    //re-power on
     while (WAITING==(ATCommand((const unsigned char *)"AT Z"))) mySerial->poll();

    delay(5);


    while (WAITING==(temp=ATCommand((const unsigned char *)"AT E 0"))) mySerial->poll();
    if (FAIL==temp)            
    {
       SuccessCurrentProtocol=false;
       return false;  
    }  
    //delay(70);
    delay(350);//200,250, 297 not good, 298:ok 350:ok,300:ok, 500:ok
    //clear " Search... cannot connect "
    while (mySerial->available()) mySerial->read();
    //Wait for "Sleep warning"

    delay(4700);  //4567: not good, 4568, 4600, 4900:ok,  stm32 :4630 not work,4640 OK
    ResponseStr=mySerial->readString();
    

    if (ResponseStr.substring(2,9)=="Warning")
    {
       SuccessCurrentProtocol=false;
       //Serial.println("I got warining");
       //return false;    //version1.0.1 modification because we want to execute "AT S 0" and "GPS Initilization"
    }
    

    TempStr=String("AT ST");
    if (Timeout<=1000){
        temp=Timeout/4;
       if (temp<=15) {
          TempStr+="0";
          TempStr+=String(temp,16);          
        }
        else {
          TempStr+=String(temp,16);            
        }
        TempStr.toCharArray(bufferx, 50);
        while (WAITING==(temp=ATCommand((const unsigned char *)bufferx))) mySerial->poll();
        if (FAIL==temp)            
        {

           SuccessCurrentProtocol=false;
           return false;  
        }
    }       
    else if (Timeout<=2000) {
        while (WAITING==(temp=ATCommand((const unsigned char *)"AT ST FB"))) mySerial->poll();
        if (FAIL==temp)  {
           SuccessCurrentProtocol=false;
           return false;  
        }
        
    } else if (Timeout<=4000) {
         while (WAITING==(temp=ATCommand((const unsigned char *)"AT ST FC"))) mySerial->poll();
         if (FAIL==temp)  {
           SuccessCurrentProtocol=false;
           return false;  
        }
    } else if (Timeout<=6000) {
        while (WAITING==(temp=ATCommand((const unsigned char *)"AT ST FD"))) mySerial->poll();
        if (FAIL==temp)  {
           SuccessCurrentProtocol=false;
           return false;  
        }
    } else if (Timeout<=8000) {
        while (WAITING==(temp=ATCommand((const unsigned char *)"AT ST FE"))) mySerial->poll();
        if (FAIL==temp)  {
           SuccessCurrentProtocol=false;
           return false;  
        }
    } else  {
        while (WAITING==(temp=ATCommand((const unsigned char *)"AT ST FF"))) mySerial->poll();
        if (FAIL==temp)  {
           SuccessCurrentProtocol=false;
           return false;  
        }
    }

    //AT S 
    do {
       while (WAITING==(temp=ATCommand((const unsigned char *)"AT S 0"))) mySerial->poll();
    } while (FAIL==temp) ;    
    
	//version1.0.1 add--start 
	//at dev1 tp 6  : GPS
	do {
		while (WAITING == (temp = ATCommand((const unsigned char *)"AT DEV1 TP 6"))) mySerial->poll();
	} while (FAIL == temp);
	//version1.0.1 add--end

    //AT DEV1 PC
    while (WAITING==(temp=ATCommand((const unsigned char *)"AT DEV1 PC"))) mySerial->poll();
    if (FAIL==temp)  {
       SuccessCurrentProtocol=false;
       return false;  
    }
        
    //Change DEV1 baud rate:  DEV1 BRD  hh
    TempStr=String("AT DEV1 BRD");
    if (4800==GPS.Baudrate) TempStr+="01";
    else if (2400==GPS.Baudrate) TempStr+="02";
    else if (9600==GPS.Baudrate) TempStr+="00";
    else {
      temp=(byte)(5000000/GPS.Baudrate);
      if (temp<=15) 
      {
          TempStr+="0";
      }
      TempStr+=String(temp,16);      
    }
    TempStr.toCharArray(bufferx, 50);
    do {
       while (WAITING==(temp=ATCommand((const unsigned char *)bufferx))) mySerial->poll();
    } while (FAIL==temp) ;     
    
    
    //Time out= 500ms for gps
    TempStr=String("AT DEV1 ST"); 
    if (500==GPS.Timeout){
      do {
          while (WAITING==(temp=ATCommand((const unsigned char *)"AT DEV1 ST 00"))) mySerial->poll();
      } while (FAIL==temp) ;  
      
    }
    else if (GPS.Timeout<=1000){
      temp=GPS.Timeout/4;
      if (temp<=15) {
          TempStr+="0";
          TempStr+=String(temp,16);          
      }
      else {
          TempStr+=String(temp,16);            
      }
      TempStr.toCharArray(bufferx, 50);
      do {
       while (WAITING==(temp=ATCommand((const unsigned char *)bufferx))) mySerial->poll();
      } while (FAIL==temp) ;   
             
      
    }       
    else if (GPS.Timeout<=2000) {
      do {
       while (WAITING==(temp=ATCommand((const unsigned char *)"AT DEV1 ST FB"))) mySerial->poll();
      } while (FAIL==temp) ; 
            
    } else if (GPS.Timeout<=4000) {
      do {
       while (WAITING==(temp=ATCommand((const unsigned char *)"AT DEV1 ST FC"))) mySerial->poll();
      } while (FAIL==temp) ; 
           
    } else if (GPS.Timeout<=6000) {
      do {
       while (WAITING==(temp=ATCommand((const unsigned char *)"AT DEV1 ST FD"))) mySerial->poll();
      } while (FAIL==temp) ;
      
    } else if (GPS.Timeout<=8000) {
      do {
       while (WAITING==(temp=ATCommand((const unsigned char *)"AT DEV1 ST FE"))) mySerial->poll();
      } while (FAIL==temp) ;
      
    } else  {
      do {
       while (WAITING==(temp=ATCommand((const unsigned char *)"AT DEV1 ST FF"))) mySerial->poll();
      } while (FAIL==temp) ; 
     
    }

    
    //at DEV1 TPT 0: Disable exit transparant by timeout
    do {
       while (WAITING==(temp=ATCommand((const unsigned char *)"AT DEV1 TPT 0"))) mySerial->poll();
    } while (FAIL==temp) ;
    
    //at dev1 tp 6  : GPS
    do {
       while (WAITING==(temp=ATCommand((const unsigned char *)"AT DEV1 TP 6"))) mySerial->poll();
    } while (FAIL==temp) ;
    
    if (!intrude)
    {
       do {
         while (WAITING==(temp=ATCommand((const unsigned char *)"AT INTRUDE 0"))) mySerial->poll();
       } while (FAIL==temp) ;       
    } 
    SuccessCurrentProtocol=true;
    
    if ((AUTO_PROTOCOL==currentProtocol)||
        (ISO15765_11_500_PROTOCOL==currentProtocol)||
        (ISO15765_29_500_PROTOCOL==currentProtocol)||
        (ISO15765_11_250_PROTOCOL==currentProtocol)||
        (ISO15765_29_250_PROTOCOL==currentProtocol)) {
        bufferx[0]='0';
        bufferx[1]='1'; 
        bufferx[2]='0';
        bufferx[3]='0'; 
        bufferx[4]='\0';
       
        while (WAITING==(temp=ISO15765.get2byte(/*char cmd[]*/bufferx,/*long &Out2byte*/MyLong))) mySerial->poll();
        if (SUCCESS==temp) //"0100" Command
        {
          if (MyLong&0x00002000) ISO15765.PID_13support=true; else ISO15765.PID_13support=false;  //PID 13 (Hex19) support, Bit31 is PID1, Bit 30 is PID2,..., Bit 0 is PID 32
          if (MyLong&0x00000008) ISO15765.PID_1DSupport=true; else ISO15765.PID_1DSupport=false;  //PID 1D (Hex29) support, Bit31 is PID1, Bit 30 is PID2,..., Bit 0 is PID 32
         
        }
        delay(140);//130 not good, 140:ok
        
    }
    
    delay(10);    
    //clear all data received
    while (mySerial->available()) mySerial->read();


    

   //SuccessCurrentProtocol=true;   
   return true;
}

byte DFL168A::getOnewireID(byte (&ID)[7])
{
  byte Ret;
  char Hex1Byte[]="0x00";
  long tempV;
  char i;

  Ret=ATCommand((const unsigned char *)"AT OW RD") ;
  if (WAITING==Ret) { mySerial->poll(); return Ret; }
  ReturnStr.trim();
  if (ReturnStr!="No Device Connected") 
  {

    for (i=0;i<7;i++)
    {
      Hex1Byte[2]=ReturnStr.charAt(2*i);
      Hex1Byte[3]=ReturnStr.charAt(2*i+1);
      if (!isHexadecimalDigit(Hex1Byte[2])) return FAIL;
      if (!isHexadecimalDigit(Hex1Byte[3])) return FAIL;
      tempV=strtol(Hex1Byte,NULL,16);
      ID[i]=(byte)tempV;
    }    
    return SUCCESS;
  }
  else
  {

    return FAIL;
  }
}

byte DFL168A::getDIN(int portNo, bool & Value)
{
  byte Ret;
  String TempStr;
  char bufferx[15];
  TempStr="AT RD";
  TempStr+=String(portNo,16);
  TempStr.toCharArray(bufferx, 15);
  Ret=ATCommand((const unsigned char *)bufferx) ;
  if (WAITING==Ret) { mySerial->poll(); return Ret; }
  ReturnStr.trim();
  //CLEAR all receive buffer
  clearRXDBuffer();
  if (ReturnStr=="1") {Value=true; return SUCCESS;}
  if (ReturnStr=="0") {Value=false;return SUCCESS;}
  return FAIL;
}


byte DFL168A::setDOUT(int portNo, bool Value)
{
  byte Ret;
  String TempStr;
  char bufferx[15];
  TempStr="AT WD";
  TempStr+=String(portNo,16);
  if (Value) TempStr+="1";
  else TempStr+="0";
  TempStr.toCharArray(bufferx, 15);
  Ret=ATCommand((const unsigned char *)bufferx) ;
  if (WAITING==Ret) { mySerial->poll(); return Ret; }
  //CLEAR all receive buffer
  clearRXDBuffer();
  return SUCCESS;
}


byte DFL168A::getAnalog(float & ADValue)
{
  byte Ret;
  Ret=ATCommand((const unsigned char *)"AT RV") ;
  if (WAITING==Ret) { mySerial->poll(); return Ret; }
  //CLEAR all receive buffer
  clearRXDBuffer();
  ReturnStr.trim();
  ADValue=ReturnStr.toFloat();  //0.0 to 999.000
  return SUCCESS;
}


bool DFL168A::setExitTransparentKey(byte EndTransparentChar)  //must after begin()
{
  byte temp;
  String TempStr;
  char bufferx[20];
  TempStr="ATDEV1EXITT";  //AT DEV1 EXIT T hh
  if (EndTransparentChar<=0x0f) TempStr+="0";
  TempStr+=String(EndTransparentChar,16);  
  TempStr.toCharArray(bufferx, 20);
  while (WAITING==(temp=ATCommand((const unsigned char *)bufferx))) mySerial->poll();
  if (SUCCESS==temp)
  {
    this->EndTransparentChar=EndTransparentChar; 
    return true;   
  }
  return false;
}

void DFL168A::beginTransparentSerial()
{
  mySerial->println((const unsigned char *)"ATDEV1TRANSP");  //AT DEV1 TransP
  mySerial->poll();
  TransparentSerialAvailable=true;
}

void DFL168A::endTransparentSerial()
{
  String myStr;
  if (TransparentSerialAvailable)
  {
    mySerial->write(EndTransparentChar);
    mySerial->poll();

	//delay(1);
    delay(5);  //STM 32 is fast , so 1 changed to 5
	myStr = mySerial->readString();
	myStr.trim();
	if ('>' == myStr.charAt(myStr.length() - 1))
	{
		TransparentSerialAvailable = false;
	}

  }
  
}

bool DFL168A::serialPortAvailable()
{
  return TransparentSerialAvailable;
}

 void DFL168A::clearRXDBuffer()
 {

   while (mySerial->available()) { mySerial->read();  mySerial->poll();}
 }

void DFL168A::end(void)
{

  mySerial->end();
}

//version1.0.1 add--start
bool DFL168A::setSleepDelay(unsigned int SleepDelayms)
{
	String TempStr, TemStr2;
	char myCMD[18];
	byte temp;
	TemStr2 = String(SleepDelayms, 16);
	switch (TemStr2.length())
	{
	case 1:
		TemStr2 = "000" + TemStr2;
		break;
	case 2:
		TemStr2 = "00" + TemStr2;
		break;
	case 3:
		TemStr2 = "0" + TemStr2;
		break;
	case 4:
		break;
	default:
		TemStr2 = "FFFF";
	}
	TempStr = String("AT Sleep") + TemStr2;
	TempStr.toCharArray(myCMD, 18);

	while (WAITING == (temp = ATCommand((const unsigned char *)myCMD))) mySerial->poll();

	if (SUCCESS == temp)
		return true;
	else
		return false;
}
//version1.0.1 add--End
